'use strict';

exports.__esModule = true;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _transformPathRelative = require('./transform-path-relative');

var _transformPathRelative2 = _interopRequireDefault(_transformPathRelative);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * NAMESPACE TRANSFORM
 * ===================
 */


var debug = (0, _debug2.default)('babel:plugin:namespace:transform-namespace');

/**
 * Translate the module namespace
 *
 * @param {String} sourceModule
 * @param {String} sourceFile
 * @param {Object} filesMap
 *
 * @return {String}
 */
var transformModuleNamespace = function transformModuleNamespace(sourceModule, sourceFile, filesMap) {
    // should skip if the path is relative or absolute
    if (sourceModule && (_path2.default.isAbsolute(sourceModule) || /^(\.)/.test(sourceModule))) {
        return null;
    }

    var packageConfig = (0, _utils.getPackageConfig)();

    // TODO: Replace with "signDirectoryExpansion"
    if (/^(:|~)/.test(sourceModule)) {
        sourceModule = _path2.default.join( // eslint-disable-line no-param-reassign
        packageConfig.name, sourceModule.substr(1));
    }

    var moduleSplit = sourceModule.split('/');
    var sourceModulePath = void 0;

    if (!moduleSplit.length) {
        return null;
    }

    debug('Start to map a module alias: "%s"', sourceModule);

    /**
     * This loop will check the sources map in reverse mode
     */
    while (moduleSplit.length) {
        var module = moduleSplit.join('/');

        if (filesMap.hasOwnProperty(module)) {
            sourceModulePath = filesMap[module];

            break;
        }

        moduleSplit.pop();
    }

    // No mapping available
    if (!moduleSplit.length || !sourceModulePath) {
        debug('Module alias not found: "%s"', sourceModule);

        return null;
    }

    var modulePath = moduleSplit.join('/');

    if (Array.isArray(sourceModulePath)) {
        var isPathFound = false;

        sourceModulePath.forEach(function (sourcePath) {
            var newPath = sourceModule.replace(modulePath, sourcePath);

            if ((0, _utils.isPathExists)(sourcePath) && (0, _utils.isPathExists)(newPath)) {
                modulePath = newPath;
                isPathFound = true;

                return;
            }
        });

        if (!isPathFound) {
            debug('Module: "%s" (not be found)', sourceModule);

            return null;
        }
    } else {
        modulePath = sourceModule.replace(modulePath, sourceModulePath);
    }

    debug('Module alias: "%s" ("%s")', sourceModule, modulePath);

    return (0, _transformPathRelative2.default)(sourceFile, modulePath);
};

exports.default = transformModuleNamespace;
module.exports = exports['default'];